// Application class implementation
#include "pch.h"
#include "Application.h"
#include "resources.h"

// Define window message to update the timer display
#define WM_UPDATETIME WM_USER

// Enable visual styles
#pragma comment(linker, "\"/manifestdependency:type='win32' \
name='Microsoft.Windows.Common-Controls' version='6.0.0.0' \
processorArchitecture='*' publicKeyToken='6595b64144ccf1df' language='*'\"")

namespace mainspring {
    Application::Application(const char* appName, const char* appVersion)
        : m_appName(appName)
        , m_appVersion(appVersion)
        , m_timing(false)
        , m_elapsedSaved()
    {
        std::cout << std::format("{} {}\n", m_appName, m_appVersion);
    }

    void Application::run() {
        // Read last time here

        DialogBoxParam(nullptr, MAKEINTRESOURCE(IDD_DIALOGMAIN), nullptr, reinterpret_cast<DLGPROC>(mainDlgProc), reinterpret_cast<LPARAM>(this));
    }

    BOOL CALLBACK Application::mainDlgProc(HWND dlg, UINT msg, WPARAM wParam, LPARAM lParam) {
        static Application* app = nullptr;

        switch (msg) {
            case WM_INITDIALOG: {
                app = reinterpret_cast<Application*>(lParam);

                SetWindowText(dlg, app->m_appName);
                SendMessage(dlg, WM_SETICON, ICON_BIG, reinterpret_cast<LPARAM>(LoadIcon(GetModuleHandle(NULL), MAKEINTRESOURCE(IDI_ICONMAIN))));

                POINT cursor = {};
                GetCursorPos(&cursor);
                MONITORINFO mi = {};
                mi.cbSize = sizeof(mi);
                GetMonitorInfo(MonitorFromPoint(cursor, MONITOR_DEFAULTTONEAREST), &mi);
                SetWindowPos(dlg, nullptr, mi.rcMonitor.left + 100, mi.rcMonitor.top + 100, 0, 0, SWP_NOZORDER | SWP_NOSIZE);

                SetDlgItemText(dlg, IDC_STATICTITLE, app->m_appName);
                LOGFONT lFont = {};
                lFont.lfHeight = 30;
                HFONT titleFont = CreateFontIndirect(&lFont);
                SendMessage(GetDlgItem(dlg, IDC_STATICTITLE), WM_SETFONT, reinterpret_cast<WPARAM>(titleFont), NULL);

                std::memcpy(lFont.lfFaceName, "Consolas", 9);
                lFont.lfHeight = 60;
                HFONT timeFont = CreateFontIndirect(&lFont);
                SendMessage(GetDlgItem(dlg, IDC_STATICTIME), WM_SETFONT, reinterpret_cast<WPARAM>(timeFont), NULL);

                SetDlgItemText(dlg, IDC_BUTTONSTARTPAUSE, "Start");

                SendMessage(dlg, WM_UPDATETIME, NULL, NULL);

                return TRUE;
            }

            case WM_COMMAND:
                switch (LOWORD(wParam)) {
                    case IDC_BUTTONSTARTPAUSE:
                        if (!app->m_timing) {
                            SetDlgItemText(dlg, IDC_BUTTONSTARTPAUSE, "Pause");
                            app->m_startTime = Clock::now();
                            // Timer to update time
                            SetTimer(dlg, 1, 50, nullptr);
                        }
                        else {
                            KillTimer(dlg, 1);
                            app->m_elapsedSaved += app->getElapsed();
                            SetDlgItemText(dlg, IDC_BUTTONSTARTPAUSE, !app->m_elapsedSaved ? "Start" : "Resume");
                        }

                        app->m_timing = !app->m_timing;

                        return TRUE;

                    case ID_HELP_ABOUT:
                        MessageBox(dlg, std::format("{} {}\nCopyright Grayson Riffe 2025\ngraysonriffe.com", app->m_appName, app->m_appVersion).c_str(), std::format("About {}", app->m_appName).c_str(), MB_OK);
                        return TRUE;
                }

                return FALSE;

            case WM_TIMER:
                SendMessage(dlg, WM_UPDATETIME, NULL, NULL);
                return TRUE;

            case WM_UPDATETIME: {
                Seconds totalTime = app->m_elapsedSaved + app->getElapsed();

                uint32_t hr = totalTime / (3600), min = totalTime % (3600) / 60, sec = totalTime % 60;
                SetDlgItemText(dlg, IDC_STATICTIME, std::format("{:02}:{:02}:{:02}", hr, min, sec).c_str());

                return TRUE;
            }

            case WM_CLOSE:
                // TODO: Ask to / save time here
                EndDialog(dlg, 0);
                return TRUE;
        }

        return FALSE;
    }

    Seconds Application::getElapsed() {
        if (m_timing)
            return Duration(Clock::now() - m_startTime).count();
        else
            return 0;
    }
}